const puppeteer = require('puppeteer');

// In-memory cache of rendered pages. Note: this will be cleared whenever the
// server process stops. If you need true persistence, use something like
// Google Cloud Storage (https://firebase.google.com/docs/storage/web/start).
const RENDER_CACHE = new Map();

async function ssr(url, browserWSEndpoint) {
    if (RENDER_CACHE.has(url)) {
        return { html: RENDER_CACHE.get(url), ttRenderMs: 0 };
    }

    console.info('Connecting to existing Chrome instance.');
    const start = Date.now();

    //  const browser = await puppeteer.launch();
    const browser = await puppeteer.connect({ browserWSEndpoint });
    const page = await browser.newPage();

    // 屏蔽图片之类的请求
    // 1. Intercept network requests.
    await page.setRequestInterception(true);
    page.on('request', req => {
        // 2. Ignore requests for resources that don't produce DOM
        // (images, stylesheets, media).
        const allowlist = ['document', 'script', 'xhr', 'fetch'];
        if (!allowlist.includes(req.resourceType())) {
            return req.abort();
        }

        // 3. Pass through all other requests.
        req.continue();
    });

    try {
        // networkidle0 waits for the network to be idle (no requests for 500ms).
        // The page's JS has likely produced markup by this point, but wait longer
        // if your site lazy loads, etc.
        await page.goto(url, {
            timeout: 99999 * 1000,
            waitUntil: 'domcontentloaded'
        });
        await page.waitForSelector('html'); // ensure #posts exists in the DOM.
    } catch (err) {
        console.error(err);
        throw new Error('page.goto/waitForSelector timed out.');
    }

    const html = await page.content(); // serialized HTML of page DOM.
    // await browser.close();
    await page.close();

    const ttRenderMs = Date.now() - start;
    console.info(`Headless rendered page in: ${ttRenderMs}ms`);

    RENDER_CACHE.set(url, html); // cache rendered page.

    return { html, ttRenderMs };
}

module.exports = ssr